CREATE TABLE SYS_TRANSLATION
(
  ID NUMBER(12, 0) NOT NULL,
  OWNER_ID NUMBER(12, 0) NOT NULL,
  LANGUAGE VARCHAR2(10 CHAR) NOT NULL,
  PAGE_ID NUMBER(12, 0),
  PAGE_ITEM_ID NUMBER(12, 0),
  KEY VARCHAR2(4000 CHAR) NOT NULL,
  TRANSLATION VARCHAR2(4000 CHAR) NOT NULL,
  TRANSLATED VARCHAR2(1 CHAR),
  CRE_USER VARCHAR2(100 CHAR) NOT NULL,
  CRE_TIME DATE NOT NULL,
  MOD_USER VARCHAR2(100 CHAR),
  MOD_TIME DATE,
  VERSION NUMBER(10, 0) NOT NULL
, CONSTRAINT TRANSLATION_PK PRIMARY KEY
  (
    ID
  )
  ENABLE
)
  TABLESPACE "USERS"
  LOGGING 
  PCTFREE 10
  INITRANS 1
  MAXTRANS 255
  STORAGE
  (
    INITIAL 64K
    MINEXTENTS 1
    MAXEXTENTS 2147483645
    BUFFER_POOL DEFAULT
  )
;

ALTER TABLE SYS_TRANSLATION
ADD CONSTRAINT TRANSLATION_TRANSLATED_CH CHECK
(TRANSLATED IN ('Y', 'N'))
 ENABLE
;

CREATE UNIQUE INDEX TRANSLATION_KEY_UI ON SYS_TRANSLATION (OWNER_ID, LANGUAGE, KEY);

CREATE OR REPLACE SYNONYM TRANSLATION FOR SYS_TRANSLATION;

CREATE SEQUENCE TRANSLATION_SEQ INCREMENT BY 1 MAXVALUE 999999999999999999999999999 MINVALUE 1 CACHE 20 ;


CREATE OR REPLACE TRIGGER "TBIU_TRANSLATION" 
  BEFORE INSERT OR UPDATE ON "SYS_TRANSLATION" 
  REFERENCING OLD AS OLD NEW AS NEW 
  FOR EACH ROW 
DECLARE
  l_curr_user           sys_translation.cre_user%TYPE;
BEGIN
  l_curr_user := NVL(apex_application.g_user, USER);
  
  IF inserting THEN
    IF :NEW.ID IS NULL THEN
      SELECT TRANSLATION_SEQ.NEXTVAL
      INTO :NEW.ID
      FROM dual;
    END IF;
    
    :NEW.CRE_USER := l_curr_user;
    :NEW.CRE_TIME := SYSDATE;
    :NEW.VERSION := 0;
  ELSE
    :NEW.MOD_USER := l_curr_user;
    :NEW.MOD_TIME := SYSDATE;
    :NEW.VERSION := :OLD.VERSION+1;
  END IF;
END tbi_translation;
/
