package com.swop;

import com.swop.config.ApplicationStatics;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.Enumeration;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMFactory;
import org.jdom.input.SAXBuilder;
import org.jdom.input.DOMBuilder;
import org.jdom.output.XMLOutputter;
import org.jdom.output.Format;

import java.io.File;
import java.io.IOException;

public class DomApexTranslationParser
{
  private static Logger logger  = Logger.getLogger(DomApexTranslationParser.class); // log4j logger definition

  private String targetLang;
  private HashMap appProperties;
  
  /**
   * whether th id of the source-xml-node should be used as translation key
   * instead of the source-text. Defaults to "false". <br/>
   * This is initialized by the config-property: 
           #
           # import mode for the translation key:
           #
           #  text: the text of the source is used as key (Default)
           #  id:  the internal id of the source is used as key
           #
           translationKey=id
   */
  private boolean useIdAsKey =false;
  
  public DomApexTranslationParser()
  {
    // read configuration from file
    appProperties=this.readProperties();
    
    String keyMode=(String) this.appProperties.get(ApplicationStatics.PROP_KEY_KEY_MODE);
    this.useIdAsKey = ApplicationStatics.PROP_KEY_MODE_ID.equalsIgnoreCase(keyMode);
  }
    
  private HashMap parseDoc(String pFileName)
  {
      HashMap translationMap=new HashMap();
      try
      {
        JDOMParser parser = new JDOMParser();
        // parser.parse(args[0]);
        parser.parse(pFileName);
        Document document = parser.getJDomDocument(pFileName);
        logger.debug("Resulting Document: "+document);
        Element root=document.getRootElement();
        logger.debug("root="+root.getName());
        List rootChildren=root.getChildren();
        // search for body element...
        Element child=null;
        for(int i=0; i<rootChildren.size(); i++)
        {
          child=(Element) rootChildren.get(i);
          logger.debug("child found="+child.getName());
          if(child!=null && child.getName().equalsIgnoreCase("body"))
          {
            logger.debug("body found ! break.");
            break;
          }
          else if(child!=null && child.getName().equalsIgnoreCase("file"))
          {
            this.targetLang=child.getAttribute("target-language").getValue();
            logger.debug("language found="+this.targetLang);
          }
        }
        
        Element body=child.getChild("body");
        if(body==null)
        {
          logger.error("body is null ! exit...");
          System.exit(-1);
        }
        else
        {
          logger.debug("body found !");
        }
        List nodeList=body.getChildren();
        if(nodeList!=null && nodeList.size()>0)
        {
          logger.debug("trans-unit found ! iterate...");
          for(int i=0; i<nodeList.size(); i++)
          {
            Element transNode=(Element) nodeList.get(i);
            
            List childNodes=transNode.getChildren();
            logger.debug("map-len="+childNodes.size()+" for node="+transNode.getName());
            //NodeList childNodes=transNodes.getChildNodes();
            if(childNodes!=null && childNodes.size()>0)
            {
              Element sourceNode=null;
              Element targetNode=null;
              String  transUnitId=transNode.getAttributeValue("id");
              for(int j=0; j<childNodes.size(); j++)
              {
                Element childNode=(Element) childNodes.get(j);
                if(childNode!=null)
                  logger.debug("node="+childNode.getName()+" val="+childNode.getValue());
                else
                  logger.debug("node is null !");
                  
                if(childNode.getName().equalsIgnoreCase("source"))
                  sourceNode=childNode;
                if(childNode.getName().equalsIgnoreCase("target"))
                  targetNode=childNode;                  
                  
                if(childNode.getName().equalsIgnoreCase("source") || childNode.getName().equalsIgnoreCase("target"))
                  logger.debug("child: name="+childNode.getName()+" val="+childNode.getValue());
              }
              
              String key=null;
              if(this.useIdAsKey) 
                  key=transUnitId;
              else if(sourceNode!=null) {
                  key=sourceNode.getValue().replaceAll("'", "''");
                  key=convertSpecialCharsFromApex(key);
              }
              

              if(key!=null && targetNode!=null)
              {
                Translation trans=new Translation();
                logger.debug("try to replace string <"+sourceNode.getValue()+">");
                if(transUnitId!=null && transUnitId.length()>1)
                    trans.setKey(key);
                if(targetNode.getValue()!=null && targetNode.getValue().length()>1)
                  trans.setTranslationString(targetNode.getValue().replaceAll("'", "''"));
                translationMap.put(trans.getKey(), trans);
              }
              else
                logger.error("sourceNode or targetNode is null !!!");
            }
          }
        }
      }
      catch(Exception ex)
      {
        ex.printStackTrace();
      }
      
      if(translationMap!=null && translationMap.size()<=0)
        return null;
        
      return translationMap;
  }

  private void createTranslationDoc(String pInpFileName, String pOutFileName, HashMap pTranslations)
  {
      try
      {
        JDOMParser parser = new JDOMParser();
        // parser.parse(args[0]);
        parser.parse(pInpFileName);
        Document document = parser.getJDomDocument(pInpFileName);
        logger.debug("Resulting Document: "+document);
        Element root=document.getRootElement();
        logger.debug("root="+root.getName());
        List rootChildren=root.getChildren();
        // search for body element...
        Element child=null;
        for(int i=0; i<rootChildren.size(); i++)
        {
          child=(Element) rootChildren.get(i);
          logger.debug("child found="+child.getName());
          if(child!=null && child.getChild("body")!=null)
            break;
          else if(child!=null && child.getName().equalsIgnoreCase("file"))
          {
            this.targetLang=child.getAttribute("target-language").getValue();
            logger.debug("language found="+this.targetLang);
          }
        }
        Element body=child.getChild("body");
        if(body==null)
        {
          logger.error("body is null ! exit...");
          System.exit(-1);
        }
        else
        {
          logger.debug("body found !");
        }
        List nodeList=body.getChildren();
        if(nodeList!=null && nodeList.size()>0)
        {
          logger.debug("trans-unit found ! iterate...");
          for(int i=0; i<nodeList.size(); i++)
          {
            Element transNodes=(Element) nodeList.get(i);
            
            List childNodes=transNodes.getChildren();
            logger.debug("map-len="+childNodes.size()+" for "+transNodes.getName());
            //NodeList childNodes=transNodes.getChildNodes();
            if(childNodes!=null && childNodes.size()>0)
            {
              Element sourceNode=null;
              Element targetNode=null;
              String  transUnitId=transNodes.getAttributeValue("id");
              for(int j=0; j<childNodes.size(); j++)
              {
                Element childNode=(Element) childNodes.get(j);
                if(childNode.getName().equalsIgnoreCase("source"))
                  sourceNode=childNode;
                if(childNode.getName().equalsIgnoreCase("target"))
                  targetNode=childNode;              
                  
                if(childNode.getName().equalsIgnoreCase("source") || childNode.getName().equalsIgnoreCase("target"))
                  logger.debug("child: name="+childNode.getName()+" val="+childNode.getValue());
              }
              
              String key=null;
              if(this.useIdAsKey) 
                key=transUnitId;
              else if(sourceNode!=null)
                key= convertSpecialCharsFromApex(sourceNode.getValue());              
              
              if(key!=null && targetNode!=null)
              {
                if(key!=null)
                {
                  Translation trans=(Translation) pTranslations.get(key);
                  if(trans!=null) {
                    logger.debug("translation found in file key="+trans.getKey()+" translation="+trans.getTranslationString());
                    targetNode.setText(trans.getTranslationString());
                  }
                }
                else
                  logger.debug("translation is null !");
              }
              else
                logger.error("sourceNode or targetNode is null !!!");
            }
          }
        }
        parser.writeXmlOutput(document, pOutFileName);
      }
      catch(Exception ex)
      {
        ex.printStackTrace();
      }
  }
  
  public void importTranslation(String pFileName)
  {
    String newKey = null;
    String newTranslation = null;
    
    /* This part read the translation file and store it in database !!!
    */
    HashMap translationMap=parseDoc(pFileName);
    
    TranslationDAO transDao=new TranslationDAO();
      
    if(translationMap!=null && translationMap.size()>0)
    {
      Iterator iter=translationMap.values().iterator();
      while(iter.hasNext())
      {
        Translation tra=(Translation) iter.next();
        logger.debug("key="+tra.getKey()+" value="+tra.getTranslationString());
        newKey = tra.getKey();
        newTranslation = tra.getTranslationString();
        if(newKey!=null && newKey.length()>0 && newTranslation!=null && newTranslation.length()>0)
            transDao.storeTranslation((String)this.appProperties.get(ApplicationStatics.PROP_KEY_JDBC_DRIVER), (String)this.appProperties.get(ApplicationStatics.PROP_KEY_JDBC_URL), (String)this.appProperties.get(ApplicationStatics.PROP_KEY_APP_USER), (String)this.appProperties.get(ApplicationStatics.PROP_KEY_APP_PWD), (this.targetLang==null?Translation.LANG_ENGLISH:this.targetLang), newKey, newTranslation);
      }
    }
    transDao.closeConnection();
  }
  
  private void setTargetLanguage(String pOrigFileName)
  {
      try
      {
        JDOMParser parser = new JDOMParser();
        // parser.parse(args[0]);
        parser.parse(pOrigFileName);
        Document document = parser.getJDomDocument(pOrigFileName);
        logger.debug("Resulting Document: "+document);
        Element root=document.getRootElement();
        logger.debug("root="+root.getName());
        List rootChildren=root.getChildren();
        // search for body element...
        Element child=null;
        for(int i=0; i<rootChildren.size(); i++)
        {
          child=(Element) rootChildren.get(i);
          logger.debug("child found="+child.getName());
          if(child!=null && child.getName().equalsIgnoreCase("file"))
          {
            this.targetLang=child.getAttribute("target-language").getValue();
            logger.debug("language found="+this.targetLang);
            break;
          }
        }
      }
      catch(Exception ex)
      {
        ex.printStackTrace();
      }
  }
  
  public void exportTranslation(String pOrigFileName, String pDestFileName)
  {
    this.setTargetLanguage(pOrigFileName);
    TranslationDAO transDao=new TranslationDAO();
    HashMap translationMap=transDao.getTranslationsForLanguage((String)this.appProperties.get(ApplicationStatics.PROP_KEY_JDBC_DRIVER), (String)this.appProperties.get(ApplicationStatics.PROP_KEY_JDBC_URL), (String)this.appProperties.get(ApplicationStatics.PROP_KEY_APP_USER), (String)this.appProperties.get(ApplicationStatics.PROP_KEY_APP_PWD), this.targetLang);
    if(translationMap==null)
    {
      logger.debug("No translations found in database !");
      System.exit(0);
    }
      
    createTranslationDoc(pOrigFileName, pDestFileName, translationMap);
    transDao.closeConnection();
  }
  
  private HashMap readProperties()
  {
    HashMap returnMap=new HashMap();
    
    // First read property file.
    try 
    { 
      logger.addAppender(ApplicationStatics.LOG4J_APPENDER);
        
      // clone properties to hashmap of same structure
      ApplicationProperties props = ApplicationProperties.load(ApplicationStatics.PROPFILE);
      String appUser    = props.getProperty("appUser");
      String appPwd     = props.getProperty("appPwd");
      String appHost    = props.getProperty("appHost");
      String jdbcUrl    = props.getProperty("jdbcUrl");
      String jdbcDriver = props.getProperty("jdbcDriver");
      String keyMode    = props.getProperty(ApplicationStatics.PROP_KEY_KEY_MODE);
      // set log level from property-file.
      String loglevel = props.getProperty("Loglevel");
      logger.setLevel(this.initLogLevel(loglevel));
      logger.debug("user:        " + appUser);	
      logger.debug("Password:    " + appPwd);	
      logger.debug("Host:        " + appHost);	
      logger.debug("JDBC-Url:    " + jdbcUrl);	
      logger.debug("JDBC-Driver: " + jdbcDriver);	
      
      returnMap.put(ApplicationStatics.PROP_KEY_APP_USER, appUser);
      returnMap.put(ApplicationStatics.PROP_KEY_APP_PWD, appPwd);
      returnMap.put(ApplicationStatics.PROP_KEY_APP_HOST, appHost);
      returnMap.put(ApplicationStatics.PROP_KEY_JDBC_URL, jdbcUrl);
      returnMap.put(ApplicationStatics.PROP_KEY_JDBC_DRIVER, jdbcDriver);
      returnMap.put(ApplicationStatics.PROP_KEY_KEY_MODE, keyMode);
    }
    catch (Exception ex) {
      logger.error("> Property Error: " + ex);
    }//try
    
    return returnMap;
  }

	// -----------------------------------------------------------------------------------------------------
	// initiate log level for sap property file definition
	// -----------------------------------------------------------------------------------------------------
  private Level initLogLevel(String pLogLevel)
  {
    if(pLogLevel.equalsIgnoreCase("DEBUG"))
      return Level.DEBUG;
    else if(pLogLevel.equalsIgnoreCase("INFO"))
      return Level.INFO;
    else if(pLogLevel.equalsIgnoreCase("WARN"))
      return Level.WARN;
    else if(pLogLevel.equalsIgnoreCase("ERROR"))
      return Level.ERROR;
    else
      return Level.ERROR;
  }
  
    /**
     * for a correct handling convert special characters.
     */
    private String convertSpecialCharsFromApex(String pVal) {
        logger.debug("convertSpecialCharsFromApex: begin with value <"+pVal+">");
        if(pVal==null) {
            logger.debug("convertSpecialCharsFromApex: value is null ! return.");
            return null;
        }
        
        String retVal=pVal;

        retVal=retVal.replaceAll("<",   "#LT#");
        retVal=retVal.replaceAll(">",   "#GT#");
        retVal=retVal.replaceAll("/",   "#SL#");
        retVal=retVal.replaceAll("&",   "#AMP#");
        retVal=retVal.replaceAll("\"",  "#DQ#");
        retVal=retVal.replaceAll(" ",   "#WS#");
        retVal=retVal.replaceAll("$",   "#DOL#");
        retVal=retVal.replaceAll("%",   "#PCT#");
        retVal=retVal.replaceAll("\n",  "#LB#");

        retVal=retVal.trim();
        
        if(retVal.length()>4000)
            retVal=retVal.substring(4000);
        
        logger.debug("convertSpecialCharsFromApex: end. return <"+retVal+">");
        return retVal;
    }

    /**
     * revert special characters from translation to the correct form for apex.
     */
    private String revertSpecialCharsForApex(String pVal) {
        logger.debug("revertSpecialCharsFromKey: begin with value <"+pVal+">");
        if(pVal==null) {
            logger.debug("revertSpecialCharsFromKey: value is null ! return.");
            return null;
        }
        
        String retVal=pVal;

        retVal=retVal.replaceAll("#LT#",   "<");
        retVal=retVal.replaceAll("#GT#",   ">");
        retVal=retVal.replaceAll("#SL#",   "/");
        retVal=retVal.replaceAll("#AMP#",  "&");
        retVal=retVal.replaceAll("#DQ#",   "\"" );
        retVal=retVal.replaceAll("#WS#",   " ");
        retVal=retVal.replaceAll("#DOL#",  "$");
        retVal=retVal.replaceAll("#PCT#",  "%");
        retVal=retVal.replaceAll("#LB#",   "\n" );

        retVal=retVal.trim();
        
        logger.debug("revertSpecialCharsFromKey: end. return <"+retVal+">");
        return retVal;
    }

  /**
   * 
   * @param args
   */
  public static void main(String[] args)
  {
    // String inFileNameDefault = "D:\\development\\src\\idispo\\app\\Apex\\src\\application\\NLS\\f1015_1016_en_de.xlf";
    // String outFileNameDefault = "D:\\development\\src\\idispo\\app\\Apex\\src\\application\\NLS\\f1015_1016_en_de_translated.xlf";
    String inFileNameDefault = "C:\\temp\\f1015_1016_en_de.xlf";
    String outFileNameDefault = "C:\\temp\\f1015_1016_en_de_translated.xlf";
    String inFileName = null;
    String outFileName = null;

    DomApexTranslationParser doc=new DomApexTranslationParser();

/*

inFileName=inFileNameDefault;
outFileName=outFileNameDefault;

doc.importTranslation(inFileName);

*/

    if(args.length < 2)
    {
      logger.error("*************************************************************************************************");
      logger.error("java DomAPexTranslationParser <import/export> <dest-filename> <target-filename (only for export)>");
      logger.error("*************************************************************************************************");
      logger.debug("... use default FILES for IMPORT !!! no EXPORT specified.");
      inFileName=inFileNameDefault;

      // read orig-translation-file and store translations into database.
      doc.importTranslation(inFileName);
    }
    else
    {
      if(args[0].equalsIgnoreCase("import"))
      {
        inFileName=args[1];
  
        // read orig-translation-file and store translations into database.
        doc.importTranslation(inFileName);
      }
      else if(args[0].equalsIgnoreCase("export"))
      {
        if(args.length<3)
        {
          logger.error("*************************************************************************************************");
          logger.error("java DomAPexTranslationParser <import/export> <dest-filename> <target-filename (only for export)>");
          logger.error("*************************************************************************************************");
        }
        else
        {
          inFileName=args[1];
          outFileName=args[2];
  
          // read translations from database and create new translation-file.
          doc.exportTranslation(inFileName, outFileName);
        }
      }
      else
      {
        logger.error("*************************************************************************************************");
        logger.error("java DomAPexTranslationParser <import/export> <dest-filename> <target-filename (only for export)>");
        logger.error("*************************************************************************************************");
      }
    }
  }
}