package com.swop;

import com.swop.config.ApplicationStatics;

import java.sql.*;

import java.util.HashMap;
import org.apache.log4j.Logger;

public class TranslationDAO 
{
  private static Logger logger  = Logger.getLogger(TranslationDAO.class); // log4j logger definition

  private Connection dbConnection;
  
  public TranslationDAO()
  {
  }

  private Connection getConnection(String jdbcDriver, String jdbcUrl, String appUser, String appPwd)
    throws Throwable
  {
    if(this.dbConnection==null || this.dbConnection.isClosed())
    {
      DriverManager.registerDriver (new oracle.jdbc.driver.OracleDriver());
      this.dbConnection=DriverManager.getConnection(jdbcUrl, appUser, appPwd);
    }

    return this.dbConnection;
  }
  
  public void closeConnection()
  {
    try
    {
      if(this.dbConnection!=null)
        this.dbConnection.close();
    }
    catch(Throwable thr)
    {
      thr.printStackTrace();
    }
  }

  public void storeTranslation(String pJdbcDriver, String pJdbcUrl, String pUsername, String pPassword, String pLang, String pKey, String pTranslation)
  {
    try
    {
      if(pLang==null|| pKey==null || pTranslation==null)
      {
        logger.error("null found !");
        return;
      }
      else {
          logger.debug("storeTranslation: key <"+pKey+">");
      }
      
      Connection con=this.getConnection(pJdbcDriver, pJdbcUrl, pUsername, pPassword);
      Statement stmt=con.createStatement();
      Translation trans=this.getTranslationsForLanguage(pJdbcDriver, pJdbcUrl, pUsername, pPassword, pLang, pKey);
      if(trans==null)
      {
        logger.debug("inserting lang="+pLang+" key="+pKey);
        stmt.executeUpdate("insert into sys_translation (owner_id, language, key, translation) values("
                      +ApplicationStatics.APPLICATION_USER_ID+", '"+pLang+"', '"+pKey+"', '"+pTranslation+"')");
        con.commit();
        logger.debug("... inserted");      
      }
      else
      {
        logger.debug("Translations exists in database ! Nothing to do.");
      }
      stmt.close();
    }
    catch(SQLException sqlEx)
    {
        System.err.println("SQLEx: storeTranslation: key <"+pKey+"> transl <"+pTranslation+">");
        logger.error("storeTranslation: key <"+pKey+"> transl <"+pTranslation+">");
        sqlEx.printStackTrace();
    }
    catch(Throwable thr)
    {
        System.err.println("Thr: storeTranslation: key <"+pKey+"> transl <"+pTranslation+">");
      logger.error("storeTranslation: key <"+pKey+"> transl <"+pTranslation+">");
      thr.printStackTrace();
    }
  }
  
  public HashMap getTranslationsForLanguage(String pJdbcDriver, String pJdbcUrl, String pUsername, String pPassword, String pLang)
  {
    HashMap retMap=new HashMap();
    try
    {
      Connection con=this.getConnection(pJdbcDriver, pJdbcUrl, pUsername, pPassword);
      Statement stmt=con.createStatement();
      logger.debug("search for translations with language="+pLang);
      String queryStr="select key, translation from sys_translation where owner_id="+ApplicationStatics.APPLICATION_USER_ID+" and language='"+pLang+"'";
      ResultSet result=stmt.executeQuery(queryStr);
      while(result.next())
      {
        String key=result.getString("key");
        String trans=result.getString("translation");
        logger.debug("readed key="+key+" trans="+trans);
        if(key!=null && trans!=null)
        {
          Translation translation=new Translation();
          translation.setKey(key);
          translation.setTranslationString(trans);
          retMap.put(key, translation);
        }
      }
      result.close();
      stmt.close();
      logger.debug("returnMap="+retMap.size());
    }
    catch(Throwable thr)
    {
      thr.printStackTrace();
    }
    if(retMap.size()<=0)
      return null;
    
    return retMap;
  }
  
  public Translation getTranslationsForLanguage(String pJdbcDriver, String pJdbcUrl, String pUsername, String pPassword, String pLang, String pKey)
    throws Throwable
  {
    Translation retTranslation=null;
    Connection con=this.getConnection(pJdbcDriver, pJdbcUrl, pUsername, pPassword);
    Statement stmt=con.createStatement();
    String queryStr="select translation from sys_translation where owner_id="+ApplicationStatics.APPLICATION_USER_ID+" and language='"+pLang+"' and key='"+pKey+"'";
    ResultSet result=stmt.executeQuery(queryStr);
    logger.debug("lang="+pLang+" key="+pKey+" size="+result.getFetchSize());
    if(result!=null && result.getFetchSize()>0 && result.next())
    {
      String key=pKey;
      String trans=result.getString("translation");
      logger.debug("readed key="+key+" trans="+trans);
      if(key!=null && trans!=null)
      {
        retTranslation=new Translation();
        retTranslation.setKey(key);
        retTranslation.setTranslationString(trans);
      }

      logger.error("Translation found for language="+pLang+" and key="+pKey);
    }

    result.close();
    stmt.close();
    
    return retTranslation;
  }
  
  /*
  public static void main(String[] args)
  {
    TranslationDAO transDao=new TranslationDAO();
    
    //transDao.storeTranslation("de", "Test java", "Test java trans");
    HashMap map=transDao.getTranslationsForLanguage(Translation.LANG_GERMAN);
  }
  */
}